<?php
/**
 * Plugin Name: eZbloo Post Export to CSV
 * Plugin URI:  https://ezbloo.com
 * Description: Export posts with customizable fields, post types, date range, categories, tags, and custom meta fields to import into your eZbloo system. You can find the export in your tools menu on the left.
 * Version:     1.3
 * Author:      Maximus McCullough
 * Author URI:  https://a1websitepro.com/
 */

if (!defined('ABSPATH')) {
    exit; // Prevent direct access
}

// Add a menu item in the WordPress admin
function custom_export_menu() {
    add_submenu_page(
        'tools.php',
        'eZbloo Export Posts to CSV',
        'eZbloo Export Posts',
        'manage_options',
        'custom-post-export',
        'custom_export_page'
    );
}
add_action('admin_menu', 'custom_export_menu');

// Function to display the export page
function custom_export_page() {
    ?>
    <div class="wrap">
        <h1>Export Posts to CSV for eZbloo</h1>
        <p>Select the options below and click "Download CSV". If you have many posts make sure that you only export a few months at a time. This will make it easier when you are importing to the eZbloo system. The file export goes from midnight to midnight, so if you want your posts from today you would select tomorrow for the export.</p>

        <form method="post" action="<?php echo esc_url(admin_url('admin-post.php')); ?>">
            <input type="hidden" name="action" value="export_posts_csv">

            <h3>Select Fields:</h3>
            <?php
            $fields = [
                'id' => 'ID',
                'user' => 'Post Author',
                'pubdate' => 'Post Date',
                'content' => 'Post Content',
                'title' => 'Post Title',
                'excerpt' => 'Post Excerpt',
                'status' => 'Post Status',
                'comment_status' => 'Comment Status',
                'slug' => 'Post Name',
                'updated' => 'Post Modified',
                'file' => 'GUID',
                'menu_order' => 'Menu Order',
                'type' => 'Post Type',
                'mime_type' => 'Post Mime Type',
                'comment_count' => 'Comment Count'
            ];

            foreach ($fields as $key => $label) {
                echo '<label><input type="checkbox" name="fields[]" value="' . esc_attr($key) . '" checked> ' . esc_html($label) . '</label><br>';
            }
            ?>

            <h3>Select Post Types:</h3>
            <?php
            $post_types = ['page', 'post', 'attachment', 'wprm_recipe', 'product'];
            foreach ($post_types as $type) {
                echo '<label><input type="checkbox" name="post_types[]" value="' . esc_attr($type) . '" checked> ' . esc_html(ucfirst($type)) . '</label><br>';
            }
            ?>

            <h3>Filter by Date Range:</h3>
            <label>From: <input type="date" name="start_date"></label>
            <label>To: <input type="date" name="end_date"></label>

            <h3>Filter by Categories:</h3>
            <select name="category" multiple>
                <?php
                $categories = get_categories();
                foreach ($categories as $category) {
                    echo '<option value="' . esc_attr($category->term_id) . '">' . esc_html($category->name) . '</option>';
                }
                ?>
            </select>

            <h3>Filter by Tags:</h3>
            <select name="tags" multiple>
                <?php
                $tags = get_tags();
                foreach ($tags as $tag) {
                    echo '<option value="' . esc_attr($tag->term_id) . '">' . esc_html($tag->name) . '</option>';
                }
                ?>
            </select>

            <h3>Select Custom Meta Fields:</h3>
            <input type="text" name="custom_meta" placeholder="Enter custom meta keys, separated by commas">

            <br><br>
            <input type="submit" class="button button-primary" value="Download CSV">
        </form>
    </div>
    <?php
}

// Function to handle CSV export request
function custom_export_to_csv() {
    if (!current_user_can('manage_options')) {
        wp_die(__('You do not have permission to access this page.'));
    }

    global $wpdb;

    // Get selected fields
    $selected_fields = isset($_POST['fields']) ? $_POST['fields'] : [];
    if (empty($selected_fields)) {
        wp_die(__('No fields selected.'));
    }

    // Get selected post types
    $selected_post_types = isset($_POST['post_types']) ? $_POST['post_types'] : ['page', 'post', 'attachment', 'wprm_recipe', 'product'];

    // Date range filtering
    $start_date = !empty($_POST['start_date']) ? $_POST['start_date'] : '0000-00-00';
    $end_date = !empty($_POST['end_date']) ? $_POST['end_date'] : date('Y-m-d');

    // Category and tag filtering
    $category_filter = !empty($_POST['category']) ? $_POST['category'] : [];
    $tag_filter = !empty($_POST['tags']) ? $_POST['tags'] : [];

    // Custom meta fields
    $custom_meta_keys = !empty($_POST['custom_meta']) ? explode(',', $_POST['custom_meta']) : [];

    // Map field names to database columns
    $field_map = [
        'id' => 'ID',
        'user' => 'post_author',
        'pubdate' => 'post_date',
        'content' => 'post_content',
        'title' => 'post_title',
        'excerpt' => 'post_excerpt',
        'status' => 'post_status',
        'comment_status' => 'comment_status',
        'slug' => 'post_name',
        'updated' => 'post_modified',
        'file' => 'guid',
        'menu_order' => 'menu_order',
        'type' => 'post_type',
        'mime_type' => 'post_mime_type',
        'comment_count' => 'comment_count'
    ];

    $columns = [];
    foreach ($selected_fields as $field) {
        if (isset($field_map[$field])) {
            $columns[] = $field_map[$field];
        }
    }

    $query = "
        SELECT " . implode(', ', $columns) . "
        FROM {$wpdb->posts}
        WHERE post_type IN ('" . implode("','", $selected_post_types) . "')
        AND post_date BETWEEN '$start_date' AND '$end_date'
    ";

    $posts = $wpdb->get_results($query, ARRAY_A);

    if (empty($posts)) {
        wp_die(__('No posts found for export.'));
    }

    // Include custom meta fields
    if (!empty($custom_meta_keys)) {
        foreach ($posts as &$post) {
            foreach ($custom_meta_keys as $meta_key) {
                $meta_value = get_post_meta($post['ID'], trim($meta_key), true);
                $post[$meta_key] = is_array($meta_value) ? json_encode($meta_value) : $meta_value;
            }
        }
    }

    // Prepare CSV headers
    header('Content-Type: text/csv; charset=utf-8');
    header('Content-Disposition: attachment; filename=exported_posts.csv');

    $output = fopen('php://output', 'w');
    fputcsv($output, array_merge(array_values($selected_fields), $custom_meta_keys));

    foreach ($posts as $post) {
        fputcsv($output, array_values($post));
    }

    fclose($output);
    exit;
}

// Hook into WordPress admin action handler
add_action('admin_post_export_posts_csv', 'custom_export_to_csv');
